(*pp camlp4o pa_ifdef.cmo `cat CONFIG` *)

(*

  CamlTemplate: A template processor for Objective Caml programs.
  Copyright  2003, 2004 Benjamin Geer
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
  USA
  
  In addition, as a special exception, Benjamin Geer gives permission
  to link the code of this program with the Apache HTTP Server (or
  with modified versions of Apache that use the same license as
  Apache), and distribute linked combinations including the two. You
  must obey the GNU General Public License in all respects for all of
  the code used other than Apache. If you modify this file, you may
  extend this exception to your version of the file, but you are not
  obligated to do so. If you do not wish to do so, delete this
  exception statement from your version.

*)

(* $Id: ctUtil.ml,v 1.9 2004/02/24 19:10:50 ben Exp $ *)

(* Miscellaneous utility functions. *)

(* Returns the contents of a filename as a string. *)
let read_as_string filename =
  (* Reads at most len chars into str, and returns the number of chars
     read. *)
  let rec read_into_str in_chan str offset len =
    if len <= 0 then offset else
      (
        let char_count = input in_chan str offset len in
          if char_count = 0
          then offset
          else read_into_str in_chan str (offset + char_count) (len - char_count)
      )
  in
  let in_chan = open_in_bin filename in
  let max_size = in_channel_length in_chan in
  let buf = String.create max_size in
  let total_char_count = read_into_str in_chan buf 0 max_size in
    close_in in_chan;
    String.sub buf 0 total_char_count ;;

(* Implements try-with-finally. *)
let run_with_finally ~f ~(finally: unit -> unit) =
  let res =
    try
      f ()
    with e ->
      finally ();
      raise e
  in
    finally ();
    res ;;

(* Locks a mutex, calls a function, then unlocks the mutex.  The mutex
   is also unlocked if the function throws an exception. *)
ifdef THREADS then
  let run_in_mutex ~f ~(mutex : Mutex.t) =
    Mutex.lock mutex;
    run_with_finally ~f ~finally:(function () -> Mutex.unlock mutex) ;;

(* If threads are enabled, creates a mutex, otherwise returns unit. *)
let create_optional_mutex () =
  (ifdef THREADS then
     Mutex.create ()
   else
     ()) ;;

let run_in_optional_mutex ~f ~mutex =
  (ifdef THREADS then
     run_in_mutex ~f ~mutex
   else
     f ()) ;;
